#include "double_connected_linked_list.h"

#include <stdio.h> // printf is needed
#include <stdlib.h>  // NULL and malloc are needed 
#include <stdbool.h>
#include <assert.h> // assert is needed

static dcll_entry_t* allocate_dcl_entry(int value);
static void insert_dcll(int value, dcll_entry_t *cur);

// - function ----------------------------------------------------------------
void free_dcll(dc_linked_list_t *list)
{
   if (list) {
      dcll_entry_t *cur = list->head;
      while(cur) {
         dcll_entry_t *prev = cur;
         cur = cur->next;
         free(prev);
      }
   }
}

// - function ----------------------------------------------------------------
void push_dcll(int value, dc_linked_list_t *list)
{
   assert(list);
   dcll_entry_t *new_entry = allocate_dcl_entry(value);
   if (list->head) { // an entry already in the list
      new_entry->next = list->head; // connect new -> head
      list->head->prev = new_entry; // connect new <- head
   } else { //list is empty
      list->end = new_entry;
   }
   list->head = new_entry; //update the head 
}

// - function ----------------------------------------------------------------
void print_dcll(const dc_linked_list_t *list)
{
   if (list && list->head) {
      dcll_entry_t *cur = list->head;
      while (cur) {
         printf("%i%s", cur->value, cur->next ? " " : "\n");
         cur = cur->next;
      }
   }
}

// - function ----------------------------------------------------------------
void printReverse(const dc_linked_list_t *list)
{
   if (list && list->end) {
      dcll_entry_t *cur = list->end;
      while (cur) {
         printf("%i%s", cur->value, cur->prev? " " : "\n");
         cur = cur->prev;
      }
   }
}

// - helper local function ---------------------------------------------------
dcll_entry_t* allocate_dcl_entry(int value)
{
   dcll_entry_t *new_entry = (dcll_entry_t*)malloc(sizeof(dcll_entry_t));
   assert(new_entry);
   new_entry->value = value;
   new_entry->next = NULL;
   new_entry->prev = NULL;
   return new_entry;
}

// - helper local function ---------------------------------------------------
// Insert value before the cur entry
void insert_dcll(int value, dcll_entry_t *cur)
{
   assert(cur);
   dcll_entry_t *new_entry = allocate_dcl_entry(value);
   new_entry->next = cur;
   new_entry->prev = cur->prev;
   if (cur->prev != NULL) {
      cur->prev->next = new_entry;
   }
   cur->prev = new_entry;
}

